// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// Copyright 2005 David le Roux

#ifndef QT3
 #include <QtGui>
#endif

#include "SudokuWidget.h"
#include "MainWindow.h"
#include <iostream>
using namespace std;

MainWindow::MainWindow()
{
	  setMinimumSize(500,500);
 
	  
	  undoAct = new QAction(tr("Undo"), this);
	  redoAct = new QAction(tr("Redo"), this);
	  
	  QWidget *centralWidget = new QWidget;

	qStatusBar = statusBar();
	
    scrollArea = new QScrollArea;
    sudokuWidget = new SudokuWidget(undoAct,redoAct,qStatusBar);
    scrollArea->setWidget(sudokuWidget);
    scrollArea->setWidgetResizable(true);


    QVBoxLayout *centralLayout = new QVBoxLayout;
    centralLayout->addWidget(scrollArea, 1);
    centralWidget->setLayout(centralLayout);

    setCentralWidget(centralWidget);
    
 
    qMenuBar = menuBar();
	  setupMenuBar();
    
}

void MainWindow::setupMenuBar()
{
	QMenu* fileMenu = new QMenu(tr("&File"));
	
	QAction* openAct = fileMenu->addAction("Open");
	connect(openAct, SIGNAL(triggered()), sudokuWidget, SLOT(openSlot()));
	QAction* saveAct = fileMenu->addAction("Save");
	connect(saveAct, SIGNAL(triggered()), sudokuWidget, SLOT(saveSlot()));
	QAction* saveAsImageAct = fileMenu->addAction("Save as &Image");
	connect(saveAsImageAct, SIGNAL(triggered()), sudokuWidget, SLOT(saveAsImageSlot()));
	
	qMenuBar->addMenu(fileMenu);
	
	QMenu* gameMenu = new QMenu(tr("&Game"));
	
	QMenu* generateMenu = gameMenu->addMenu(tr("Generate &New"));
	QMenu* threeMenu = generateMenu->addMenu(tr("3x3"));
	QMenu* fourMenu = generateMenu->addMenu(tr("4x4"));
	QMenu* fiveMenu = generateMenu->addMenu(tr("5x5"));
//	QMenu* check = threeMenu->addMenu(tr("Easy"));
	QAction* threeEasy = threeMenu->addAction(tr("Easy"));
	connect(threeEasy,SIGNAL(triggered()),sudokuWidget,SLOT(threeEasy()));
	QAction* threeMed =threeMenu->addAction(tr("Medium"));
	connect(threeMed,SIGNAL(triggered()),sudokuWidget,SLOT(threeMed()));
	QAction* threeHard =threeMenu->addAction(tr("Hard"));
	connect(threeHard,SIGNAL(triggered()),sudokuWidget,SLOT(threeHard()));
	
	QAction* fourEasy  = fourMenu->addAction(tr("Easy"));
	connect(fourEasy,SIGNAL(triggered()),sudokuWidget,SLOT(fourEasy()));
	QAction* fourMed  = fourMenu->addAction(tr("Medium"));
	connect(fourMed,SIGNAL(triggered()),sudokuWidget,SLOT(fourMed()));
	QAction* fourHard  = fourMenu->addAction(tr("Hard"));
	connect(fourHard,SIGNAL(triggered()),sudokuWidget,SLOT(fourHard()));
	
	QAction* fiveEasy = fiveMenu->addAction(tr("Easy"));
	connect(fiveEasy,SIGNAL(triggered()),sudokuWidget,SLOT(fiveEasy()));
	QAction* fiveMed = fiveMenu->addAction(tr("Medium"));
	connect(fiveMed,SIGNAL(triggered()),sudokuWidget,SLOT(fiveMed()));
	QAction* fiveHard = fiveMenu->addAction(tr("Hard"));
	connect(fiveHard,SIGNAL(triggered()),sudokuWidget,SLOT(fiveHard()));
	
//	QAction* generateAct = gameMenu->addAction(tr("Generate &New"));
//	connect(generateAct, SIGNAL(triggered()), sudokuWidget, SLOT(generateSlot()));
	QAction* showSolutionsAct = gameMenu->addAction(tr("Sho&w Solutions"));
	connect(showSolutionsAct, SIGNAL(triggered()), sudokuWidget, SLOT(showSolutionsSlot()));
	QAction* fixAct = gameMenu->addAction(tr("Fi&x Game"));
	connect(fixAct, SIGNAL(triggered()), sudokuWidget, SLOT(fixSlot()));
	QAction* clearNonFixedAct = gameMenu->addAction(tr("&Clear Non-Fixed"));
	connect(clearNonFixedAct, SIGNAL(triggered()), sudokuWidget, SLOT(clearNonFixedSlot()));
	QAction* clearAllAct = gameMenu->addAction(tr("Clear &All"));
	connect(clearAllAct, SIGNAL(triggered()), sudokuWidget, SLOT(clearAllSlot()));
	
	//QAction* digitCountAct = gameMenu->addAction(tr("&Digit Count"));
	//connect(digitCountAct, SIGNAL(triggered()), sudokuWidget, SLOT(digitCountSlot()));
	
	qMenuBar->addMenu(gameMenu);
	
	QMenu* editMenu = new QMenu(tr("Edit"));

	
//	undoAct = editMenu->addAction(tr("Undo"));
	undoAct->setShortcut(tr("Ctrl+Z"));
	connect(undoAct, SIGNAL(triggered()), sudokuWidget, SLOT(undoSlot()));
	editMenu->addAction(undoAct);
	
	
	
//	redoAct = editMenu->addAction(tr("Redo"));
	redoAct->setShortcut(tr("Ctrl+R"));
	connect(redoAct, SIGNAL(triggered()), sudokuWidget, SLOT(redoSlot()));
	editMenu->addAction(redoAct);
	
	//later, should be able to set maxSolutions, show clashes with solution ...
//	QAction* settingsAct = editMenu->addAction(tr("Settings"));
//	connect(settingsAct, SIGNAL(triggered()), sudokuWidget, SLOT(settingsSlot()));
	
	qMenuBar->addMenu(editMenu);
	
	QMenu* helpMenu = new QMenu(tr("&Help"));

	QAction* manualAct = helpMenu->addAction(tr("&Manual"));
	connect(manualAct, SIGNAL(triggered()), sudokuWidget, SLOT(manualSlot()));
	QAction* aboutAct = helpMenu->addAction(tr("&About"));
	connect(aboutAct, SIGNAL(triggered()), sudokuWidget, SLOT(aboutSlot()));
	
	qMenuBar->addMenu(helpMenu);
}
