// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// Copyright 2005 David le Roux

#include "SolutionWidget.h"

 #include <QtGui>
 #include <QImage>

#include <math.h>
#include <iostream>
using namespace std;


SolutionWidget::SolutionWidget(QLabel* l)
    : QWidget(0)
{
     label = l;
    QPalette palette;
    palette.setColor(backgroundRole(), QColor(Qt::white));
    setPalette(palette);
    
    displayFont.setFamily("Arial");

    
//    setAttribute(Qt::WA_PaintOnScreen,true);
//    setAttribute(Qt::WA_PaintOutsidePaintEvent);  
    
    side = 0;
 //   setSide(9);

//	solutionStruct = new SolutionStruct();	
}

//	vector< vector<int> > solutions;

void SolutionWidget::updateLabel()
{
	QString indexString;
	indexString.setNum(solutionIndex+1);
	QString totalString;
	totalString.setNum(solutions.size());
	label->setText("Solution "+indexString + " of "+totalString);
}
void SolutionWidget::updateGrid()
{
	for(int i=0; i<side; i++)
		for(int j=0; j<side; j++)
			grid[i][j]->fix = solutions[solutionIndex][i*side +j];
	updateLabel();
}
void SolutionWidget::setSolutions(SolutionStruct* s)
{
	solutions = s->solutions;
	int side = (int)sqrt((float)solutions[0].size());
	solutionIndex = 0;
	setSide(side);
	updateGrid();
	update();
}
void SolutionWidget::nextSlot()
{
	solutionIndex++;
	if(solutionIndex == solutions.size())
		solutionIndex = 0;
	updateGrid();
	update();
}
void SolutionWidget::setSide(int s)
{
  int root = sqrt((float)s);
  if(root*root == s)
  {
    for(int i=0; i<side; i++)
    {
      for(int j=0; j<side; j++)
      {
            delete grid[i][j];
      }  
      delete[] grid[i];
    } 
    if(side > 0)
        delete[] grid;
      
	
//	int oldSideSquared = side*side;
    side = s;
    squareRootOfSide = root;
	sideSquared = side*side;
    int minimumSquareSize;
    if(side < 10)
        minimumSquareSize = 30;
    else
        minimumSquareSize = 50;
    setMinimumSize(side*minimumSquareSize,side*minimumSquareSize);
    
    grid = new SimpleSquare**[side];
    for(int i=0; i<side; i++)
    {
      grid[i] = new SimpleSquare*[side];
      for(int j=0; j<side; j++)
      {
        grid[i][j] = new SimpleSquare(1);
      }     
    }  
    numbersAsStrings.clear();
    for(int i=1; i <= side; i++)
        numbersAsStrings.push_back(QString::number(i));
  
		update();
  }
}  
/*
void SolutionWidget::displayPuzzle(vector<int> puzzle)
{
	setSide((int)sqrt((float)puzzle.size()),false);
	for(int i=0; i<puzzle.size(); i++)
	{
		if(puzzle[i] != 0)
		{
			int row = i/side;
			int col = i - row*side;
			int num = puzzle[i]-1;
			fix(row,col,num,false);
		}
	}
	fixSlot(true);
}*/
/*
void SolutionWidget::fixSlot(bool undoDisabled)
{
  gameFixed = true;
  vector<RowCol> v;
	
  for(int i=0; i<side; i++)
    for(int j=0; j<side; j++)
    {
        if(grid[i][j]->fix > 0)
		{
			if(!undoDisabled)
				backupSquare(i,j);
            grid[i][j]->reallyFixed = true;
			if(!undoDisabled)
				v.push_back(RowCol(i,j));
			update(squareToPaint(i,j));
		}
    }
	if(!undoDisabled)
  		saveChangedSquares(v);	
}*/
QRect SolutionWidget::squareToPaint(QPoint pos)
{
  int paintx = (pos.x()/squareWidth)*squareWidth+1;
  int painty = (pos.y()/squareHeight)*squareHeight+1;
  return QRect(paintx,painty,squareWidth-2,squareHeight-2);
}  

QRect SolutionWidget::squareToPaint(int row,int col)
{
  int paintx = col * squareWidth + 1;
  int painty = row * squareHeight + 1;
  return QRect(paintx,painty,squareWidth-2,squareHeight-2);
}  

void SolutionWidget::resizeEvent ( QResizeEvent * event ) 
{
//	cout << "solution widget got a resizeEvent" << endl;
//	cout << "my width " << width() << endl;
//	cout << "my height " << height() << endl;
  calcFontSize();
}  
void SolutionWidget::calcFontSize()
{
   squareWidth = (width()-1)/side;
	float squareWidthForFont = (width()-side*2)/side;
   maxWidth = squareWidth*side;
   
   squareHeight = (height()-1)/side;
	float squareHeightForFont = (height()-side*2)/side;
   maxHeight = squareHeight*side;
   
	 bigFontSize = fontSizeForRect(squareWidthForFont,squareHeightForFont);
}  
float SolutionWidget::fontSizeForRect(float haveWidth, float haveHeight)
{
    int initsize = 5.0f;
    displayFont.setPointSizeF(initsize);
		QFontMetrics fm(displayFont);
		QString biggestNum = QString::number(side);
		QRect rect = fm.boundingRect(biggestNum);

		if((float)rect.width()/(float)rect.height() > (float)haveWidth/(float)haveHeight)
		{
			float scale = (float)haveWidth/(float)rect.width();
			return scale*initsize;
		}
		else
		{
			float scale = (float)haveHeight/(float)rect.height();
			return scale*initsize;
		}
}  
/*
QSize SolutionWidget::sizeHint() const
{
	return QSize(600,600);
}*/
void SolutionWidget::paintEvent(QPaintEvent *event)
{
	QRect rect = event->rect();
	QPainter painter(this);
    displayFont.setPointSizeF(bigFontSize);
    painter.setFont(displayFont);

    painter.setPen(QPen(Qt::gray));
    for(int i=0; i<= side; i++)
    {        
      painter.drawLine(1,i*squareHeight,maxWidth,i*squareHeight); //row
      painter.drawLine(i*squareWidth,1,i*squareWidth,maxHeight); //col
    }  
    
    painter.setPen(QPen(Qt::black));
    for(int i=0; i<= side; i+=squareRootOfSide)
    {
      painter.drawLine(1,i*squareHeight,maxWidth,i*squareHeight); //row
      painter.drawLine(i*squareWidth,1,i*squareWidth,maxHeight); //col
      
    }  
    //now figure out which blocks need redrawing
	
    int fromSquareRow = rect.top()/squareHeight;
    int toSquareRow = rect.bottom()/squareHeight;
    if(toSquareRow >= side)
        toSquareRow = side-1;
    
    int fromSquareCol = rect.left()/squareWidth;
    int toSquareCol = rect.right()/squareWidth;
    if(toSquareCol >= side)
        toSquareCol = side-1;
	
    for(int i=fromSquareRow; i < toSquareRow+1; i++) 
      for(int j=fromSquareCol; j < toSquareCol+1; j++)
      {
        int squareX = j*squareWidth + 1 ;//+ cellWidth/4; //it seems to draw quite to the left of the box
        int squareY = i*squareHeight + 1 ;//+ cellHeight;
       
       
         // if(grid[i][j]->reallyFixed)
         //   painter.setPen(QPen(Qt::green));
         // else
            painter.setPen(QPen(Qt::black));
          painter.drawText(squareX,squareY,squareWidth,squareHeight,Qt::AlignCenter,numbersAsStrings[grid[i][j]->fix-1]);   
		 
      }  
}
