// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// Copyright 2005 David le Roux

#ifndef _SUDOKUWIDGET_H_
#define _SUDOKUWIDGET_H_

#ifdef QT3
 #include <qfont.h>
 #include <qpoint.h>
 #include <qsize.h>
 #include <qstring.h>
 #include <qwidget.h>
#else
 #include <QFont>
 #include <QPoint>
 #include <QSize>
 #include <QString>
 #include <QWidget>
 #include <QStatusBar>
 #include <QTextBrowser>
#endif
#include "SudokuSolver.h"
#include "SolutionDialog.h"
#include <vector>
using namespace std;

class QMouseEvent;
class QPaintEvent;

enum State
{
  on,
  off,
  disabled
};  

struct Square
{
  int fix;
  bool reallyFixed;
  vector<State> numbers;
  Square()
  {
    fix = 0;
    reallyFixed = false;
  } 
  Square(Square* other)
  {
    fix = other->fix;
    reallyFixed = other->reallyFixed;
    numbers = other->numbers;
  }   
}; 
struct SquareArray
{
	Square** squares;
	SquareArray(int sideSquared)
	{
		squares = new Square*[sideSquared];
		for(int i=0; i<sideSquared; i++)
			squares[i] = 0;
	}
	~SquareArray()
	{
		delete[] squares;
	}
};

struct RowCol
{
	int row;
	int col;
	RowCol(int r, int c)
	{
		row = r;
		col = c;
	}
};
struct Highlight
{
  int row;
  int col;
  int cellRow;
  int cellCol;
  bool active;
  QRect rect;
  Highlight()
  {
    active = false;
    row = -1;
    col = -1;
    cellRow = -1;
    cellCol = -1;
  }  
  void set(int r, int c, int cr, int cc,int x1,int y1,int x2,int y2)
  {
    active = true;
    row = r;
    col = c;
    cellRow = cr;
    cellCol = cc;
    rect.setCoords(x1,y1,x2,y2);
  }  
};   
class SudokuWidget : public QWidget
{
    Q_OBJECT

public:
    SudokuWidget(QAction* undoAct, QAction* redoAct,QStatusBar* s);
    void setSide(int s,bool allowUndo=false);

public slots:
  
	void openSlot();
	void saveSlot();
	void saveAsImageSlot();
	
	void fixSlot();
	void actualFix(bool allowUndo,bool bypassFindSolution);
	//void generateSlot();
	void clearAllSlot();
	void clearNonFixedSlot();
	void digitCountSlot();
	void showSolutionsSlot();
	
	void undoSlot();
	void redoSlot();
	void settingsSlot();
	
	void manualSlot();
	void aboutSlot();

	void threeEasy();
	void threeMed();
	void threeHard();
	void fourEasy();
	void fourMed();
	void fourHard();
	void fiveEasy();
	void fiveMed();
	void fiveHard();


//signals:
//    void characterSelected(const QString &character);

protected:
    void mouseMoveEvent(QMouseEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void paintEvent(QPaintEvent *event);
    void resizeEvent ( QResizeEvent * event ) ;
    void leaveEvent ( QEvent * event );

private:
    QFont displayFont;
    int side;
    int squareRootOfSide;
	int sideSquared;
    int squareWidth;
    int maxWidth;
    int squareHeight;
    int maxHeight;
    int cellWidth;
    int cellHeight;
    float fontSize;
    float bigFontSize;
    void calcFontSize();
    Highlight* highlight;
//    void highlightCell(int row,int col,int cellRow,int cellCol);
    float fontSizeForRect(float haveWidth, float haveHeight);
    void fix(int row, int col, int num,bool allowUndo=true);
    QRect squareToPaint(QPoint p);
    QRect squareToPaint(int row,int col);
    vector<QString> numbersAsStrings;
    Square*** grid;
    bool gameFixed;
    
    QAction* redoAct;
    QAction* undoAct;
    vector< SquareArray* > undoStack;
	void backupSquare(int row,int col);
//	void putNewSquareOnUndoStack(int row,int col);
    void wipeOutRedos();
	void changeBoardFromStack(int position);
	void saveChangedSquares(vector<RowCol> v);
    int undoPosition;
	
	vector<int> readInFile(QString fileName,bool& properFileLayout,QString& contents);
	SudokuSolver* sudokuSolver;
	
	QString lastLocation;
	QString lastSavedLocation;
	SolutionStruct* solutionStruct;
	
	QStatusBar* statusBar;
	
	void displayPuzzle(vector<int> puzzle);
	void paintOnDevice(QPaintDevice* paintDevice,QRect rect);
	void writeOutSudokuFile(QString location);
	void displayProperFile(QString contents);
	
	void displayGenerated(GeneratedPuzzle* g);
	void statusBarMessage(SolutionStruct* s);
	void solveCurrent();
	vector<int> puzzleFromCurrent();
	
	SolutionDialog* solutionDialog;
	QTextBrowser* browser;
};
    
#endif /*_SUDOKUWIDGET_H_*/
